//---------------------------------------------------------------------------------------
// Export selected geometry
//---------------------------------------------------------------------------------------

#ifdef _WINDOWS
#ifndef WINDOWS
#define WINDOWS
#endif
#endif
#include "ac_plugin.h"

#include <stdio.h>
#include <time.h>
#include <sys/stat.h>

//---------------------------------------------------------------------------------------
// Post-export cleanup
static void ddhexportselected_cleanup( List *objlist, List *select_clone )
{
   ACObject *world = ac_get_world();
   for( List *po = objlist; po != NULL; po = po->next )
   {
      object_add_child( world, ( ACObject * )( po->data ));
   }
   for( List *po = select_clone; po != NULL; po = po->next )
   {
      object_delete(( ACObject * )( po->data ));
   }
   // Free the lists
   list_free( &select_clone );
   list_free( &objlist );
   // Finish
   find_bounding_box();
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
// Export selected geometry
static void ddhexportselected_go( int argc, const char* argv[] )
{

   if( argc < 3 )
   {
      message_dialog( "export_selected : insufficient parameters" );
      return;
   }

   // Get params
   char *exp_format;
   char *filename;
   exp_format = new char[strlen( argv[1] ) + 1];
   filename = new char[strlen( argv[2] ) + 1];
   strcpy( exp_format, argv[1] );
   strcpy( filename, argv[2] );

   // Gather all of the top-level objects
   ACObject *world = ac_get_world();
   List *objlist = NULL;
   for( List *po = ac_object_get_childrenlist( world ); po != NULL; po = po->next )
   {
      list_add_item_head( &objlist, po->data );
   }

   // Clone the selection
   List *export_data = clone_selected( FALSE );
   if( export_data == NULL || export_data->data == NULL )
   {
      message_dialog( "Please select at least one whole surface to be exported." );
      list_free( &objlist );
      return;
   }

   // Get exported file stats
   struct stat fi_old;
   bool file_exists_old = ( stat( filename, &fi_old ) == 0 );

   // Tell the user we've started
   time_t curtime;
   tm local;
   time( &curtime );
   local = *( localtime( &curtime ));
   tcl_command( "set ddhes_status \"%02d:%02d:%02d : %s %s\"", local.tm_hour, local.tm_min, local.tm_sec, "exporting", filename );

   // Remove geometry that was not selected
   for( List *po = objlist; po != NULL; po = po->next )
   {
      object_remove_child_nocleanup( world, ( ACObject * )( po->data ));
   }
   // Add the clones
   for( List *po = export_data; po != NULL; po = po->next )
   {
      object_add_child( world, ( ACObject * )( po->data ));
   }

   // Export data
   try
   {
      if( ac_exporter_write_file( exp_format, filename ) != TRUE )
      {
         tcl_command( "set ddhes_status \"%02d:%02d:%02d : %s %s\"", local.tm_hour, local.tm_min, local.tm_sec, "export failed for", filename );
         message_dialog( "The export failed.  Ensure the export filename is valid and that you have permissions to write to the file and directory." );
      }
      else
      {
         tcl_command( "set ddhes_status \"%02d:%02d:%02d : %s %s\"", local.tm_hour, local.tm_min, local.tm_sec, "exported", filename );
      }
      /*
      tcl_command( "ac3d exporter_write_file \"%s\" \"%s\"", exp_format, filename );
      struct stat fi;
      bool file_exists = ( stat( filename, &fi ) == 0 );
      if( !file_exists )
      {
         tcl_command( "set ddhes_status \"%02d:%02d:%02d : %s %s\"", local.tm_hour, local.tm_min, local.tm_sec, "export failed for", filename );
         message_dialog( "The export was not successful.  No file was written." );
      }
      else if( file_exists && file_exists_old && fi_old.st_mtime == fi.st_mtime )
      {
         time_t m = fi.st_mtime;
         tm m_time = *( localtime( &m ));
         tcl_command( "set ddhes_status \"%02d:%02d:%02d : %s %s\"", local.tm_hour, local.tm_min, local.tm_sec, "export may have failed for", filename );
         message_dialog( "export_selected : the Modified timestamp of the export did not change (%04d/%02d/%02d %02d:%02d:%02d).  The export may have been unsuccessful.\n\nCheck the file to ensure it was successfully written.  If it was not, check permissions on the file and directory.\n\nNote: this may also occur if you attempt to export twice within one second.",
                          m_time.tm_year, m_time.tm_mon, m_time.tm_mday, m_time.tm_hour, m_time.tm_min, m_time.tm_sec );
      }
      else
      {
         time( &curtime );
         local = *( localtime( &curtime ));
         tcl_command( "set ddhes_status \"%02d:%02d:%02d : %s %s\"", local.tm_hour, local.tm_min, local.tm_sec, "exported", filename );
      }
      */
   }
   catch( ... )
   {
      ddhexportselected_cleanup( objlist, export_data );
      throw;
   }

   ddhexportselected_cleanup( objlist, export_data );

}
//---------------------------------------------------------------------------------------
static void ddhexportselected_showwin()
{
   tcl_command( "ddhexportselected_dialog" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {

      char *path = ( char * )_strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/export_selected.tcl\"", path );

      ac_add_command( "ddhexportselected_showwin", ddhexportselected_showwin );
      ac_add_tools_menu_item( "Export selected...", "ac3d ddhexportselected_showwin",
                              "Export selected geometry" );
      ac_add_command_full( "ddhexportselected_go", ( void * )ddhexportselected_go,
                            3, "argv", "<texture index> <distance> <height>",
                           "Export selected geometry" );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Export selected - version 1.0 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Export selected - Export selected geometry." );
}
//---------------------------------------------------------------------------------------
